@@ -42,8 +42,11 @@
 	if ((p) == 0) { (p) = (m); } else { (m) = (p); } \
 } while (0)
 
+/* ST3 effect memory is not a bug, but it's a weird implementation and it's
+ * unlikely to be supported in anything other than ST3 (or OpenMPT).
+ */
 #define EFFECT_MEMORY(p, m) do { \
-	if (HAS_QUIRK(QUIRK_S3MPMEM)) { \
+	if (HAS_QUIRK(QUIRK_ST3BUGS)) { \
 		EFFECT_MEMORY__((p), xc->vol.memory); \
 	} else { \
 		EFFECT_MEMORY__((p), (m)); \
@@ -52,13 +55,13 @@
 
 #define EFFECT_MEMORY_SETONLY(p, m) do { \
 	EFFECT_MEMORY__((p), (m)); \
-	if (HAS_QUIRK(QUIRK_S3MPMEM)) { \
+	if (HAS_QUIRK(QUIRK_ST3BUGS)) { \
 		if ((p) != 0) { xc->vol.memory = (p); } \
 	} \
 } while (0)
 
 #define EFFECT_MEMORY_S3M(p) do { \
-	if (HAS_QUIRK(QUIRK_S3MPMEM)) { \
+	if (HAS_QUIRK(QUIRK_ST3BUGS)) { \
 		EFFECT_MEMORY__((p), xc->vol.memory); \
 	} \
 } while (0)
@@ -203,15 +206,6 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		}
 		break;
 	case FX_TONEPORTA:	/* Tone portamento */
-		if (HAS_QUIRK(QUIRK_IGSTPOR)) {
-			if (note == 0 && xc->porta.dir == 0)
-				break;
-		}
-		if (!IS_VALID_INSTRUMENT(xc->ins))
-			break;
-
-		do_toneporta(m, xc, note);
-
 		EFFECT_MEMORY_SETONLY(fxp, xc->porta.memory);
 
 		if (fxp != 0) {
@@ -219,6 +213,17 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 				xc->freq.memory = fxp;
 			xc->porta.slide = fxp;
 		}
+
+		if (HAS_QUIRK(QUIRK_IGSTPOR)) {
+			if (note == 0 && xc->porta.dir == 0)
+				break;
+		}
+
+		if (!IS_VALID_INSTRUMENT(xc->ins))
+			break;
+
+		do_toneporta(m, xc, note);
+
 		SET(TONEPORTA);
 		break;
 
@@ -246,7 +251,7 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		goto fx_volslide;
 
 	case FX_TREMOLO:	/* Tremolo */
-		EFFECT_MEMORY_SETONLY(fxp, xc->tremolo.memory);
+		EFFECT_MEMORY(fxp, xc->tremolo.memory);
 		SET(TREMOLO);
 		SET_LFO_NOTZERO(&xc->tremolo.lfo, LSN(fxp), MSN(fxp));
 		break;
@@ -264,6 +269,7 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 			xc->pan.val = fxp;
 			xc->pan.surround = 0;
 		}
+		xc->rpv = 0;	/* storlek_20: set pan overrides random pan */
 		break;
 	case FX_OFFSET:		/* Set sample offset */
 		EFFECT_MEMORY(fxp, xc->offset.memory);
@@ -369,16 +375,16 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		fxp &= 0x0f;
 		switch (fxt) {
 		case EX_F_PORTA_UP:	/* Fine portamento up */
-		    EFFECT_MEMORY(fxp, xc->fine_porta.up_memory);
-		    goto fx_f_porta_up;
+			EFFECT_MEMORY(fxp, xc->fine_porta.up_memory);
+			goto fx_f_porta_up;
 		case EX_F_PORTA_DN:	/* Fine portamento down */
-		    EFFECT_MEMORY(fxp, xc->fine_porta.down_memory);
-		    goto fx_f_porta_dn;
+			EFFECT_MEMORY(fxp, xc->fine_porta.down_memory);
+			goto fx_f_porta_dn;
 		case EX_GLISS:		/* Glissando toggle */
 			if (fxp) {
-				SET(GLISSANDO);
+				SET_NOTE(NOTE_GLISSANDO);
 			} else {
-				RESET(GLISSANDO);
+				RESET_NOTE(NOTE_GLISSANDO);
 			}
 			break;
 		case EX_VIBRATO_WF:	/* Set vibrato waveform */
@@ -565,6 +571,10 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		SET(FINE_VOLS_2);
 		xc->vol.fslide2 = -fxp;
 		break;
+	case FX_IT_BREAK:	/* Pattern break with hex parameter */
+		p->flow.pbreak = 1;
+		p->flow.jumpline = fxp;
+		break;
 
 #endif
 
