@@ -321,10 +321,12 @@ static int s3m_load(struct module_data *m, HIO_HANDLE *f, const int start)
         goto err3;
     }
 
+    /* Don't trust sfh.patnum */
     mod->pat = -1;
-    for (i = 0; i < mod->len && mod->xxo[i] < 0xff; ++i) {
-	if (mod->xxo[i] > mod->pat)
+    for (i = 0; i < mod->len; ++i) {
+	if (mod->xxo[i] < 0xfe && mod->xxo[i] > mod->pat) {
             mod->pat = mod->xxo[i];
+	}
     }
     mod->pat++;
     if (mod->pat > sfh.patnum)
@@ -363,7 +365,7 @@ static int s3m_load(struct module_data *m, HIO_HANDLE *f, const int start)
     case 1:
 	snprintf(tracker_name, 40, "Scream Tracker %d.%02x",
 		(sfh.version & 0x0f00) >> 8, sfh.version & 0xff);
-	m->quirk |= QUIRK_ST3GVOL;
+	m->quirk |= QUIRK_ST3BUGS;
 	break;
     case 2:
 	snprintf(tracker_name, 40, "Imago Orpheus %d.%02x",
@@ -382,6 +384,7 @@ static int s3m_load(struct module_data *m, HIO_HANDLE *f, const int start)
     case 5:
 	snprintf(tracker_name, 40, "OpenMPT %d.%02x",
 		(sfh.version & 0x0f00) >> 8, sfh.version & 0xff);
+	m->quirk |= QUIRK_ST3BUGS;
 	break;
     case 4:
 	if (sfh.version != 0x4100) {
