@@ -14,7 +14,7 @@
 double	note_to_period		(int, int, int, double);
 int	note_to_period_mix	(int, int);
 int	period_to_note		(int);
-int	period_to_bend		(double, int, int, int, double);
+int	period_to_bend		(double, int, int, double);
 void	c2spd_to_note		(int, int *, int *);
 
 #endif /* LIBXMP_PERIOD_H */
