@@ -475,6 +475,10 @@ static void process_volume(struct context_data *ctx, int chn, int act)
 	finalvol = xc->volume;
 #endif
 
+	if (m->read_event_type == READ_EVENT_IT) {
+		finalvol = xc->volume * (100 - xc->rvv) / 100;
+	}
+
 	if (TEST(TREMOLO)) {
 		finalvol += get_lfo(ctx, &xc->tremolo.lfo, 0) / (1 << 6);
 		if (!is_first_frame(ctx) || HAS_QUIRK(QUIRK_VIBALL)) {
@@ -486,7 +490,7 @@ static void process_volume(struct context_data *ctx, int chn, int act)
 
 	finalvol = (finalvol * xc->fadeout) >> 6;	/* 16 bit output */
 
-	if (HAS_QUIRK(QUIRK_ST3GVOL)) {
+	if (HAS_QUIRK(QUIRK_ST3BUGS)) {
 		gvol = 0x40;
 	} else {
 		gvol = p->gvol;
@@ -612,10 +616,38 @@ static void process_frequency(struct context_data *ctx, int chn, int act)
 
 	/* Pitch bend */
 
+
+ 	/* From OpenMPT PeriodLimit.s3m:
+	 * "ScreamTracker 3 limits the final output period to be at least 64,
+	 *  i.e. when playing a note that is too high or when sliding the
+	 *  period lower than 64, the output period will simply be clamped to
+	 *  64. However, when reaching a period of 0 through slides, the
+	 *  output on the channel should be stopped."
+	 */
+	/* ST3 uses periods*4, so the limit is 16. Adjusted to the exact
+	 * A6 value because we compute periods in floating point.
+	 */
+	if (HAS_QUIRK(QUIRK_ST3BUGS)) {
+		if (period < 16.236899) {	/* A6 */
+			period = 16.236899;
+		}
+	}
+
 	linear_bend = period_to_bend(period + vibrato, xc->note,
-			TEST(GLISSANDO) || (HAS_QUIRK(QUIRK_FT2BUGS) && arp),
 			HAS_QUIRK(QUIRK_LINEAR), xc->per_adj);
 
+	if (TEST_NOTE(NOTE_GLISSANDO) && TEST(TONEPORTA)) {
+		if (linear_bend > 0) {
+			linear_bend = (linear_bend + 6400) / 12800 * 12800;
+		} else if (linear_bend < 0) {
+			linear_bend = (linear_bend - 6400) / 12800 * 12800;
+		}
+	}
+
+	if (HAS_QUIRK(QUIRK_FT2BUGS) && arp) {
+		linear_bend = linear_bend / 12800 * 12800;
+	}
+
 	/* Envelope */
 
 	if (xc->f_idx >= 0 && (~instrument->fei.flg & XMP_ENVELOPE_FLT)) {
@@ -730,14 +762,18 @@ static void process_pan(struct context_data *ctx, int chn, int act)
 	finalpan = xc->pan.val + panbrello + (pan_envelope - 32) *
 				(128 - abs(xc->pan.val - 128)) / 32;
 
+	if (m->read_event_type == READ_EVENT_IT) {
+		finalpan = finalpan + xc->rpv * 4;
+	}
+
+	CLAMP(finalpan, 0, 255);
+
 	if (s->format & XMP_FORMAT_MONO || xc->pan.surround) {
 		finalpan = 0;
 	} else {
 		finalpan = (finalpan - 0x80) * s->mix / 100;
 	}
 
-	CLAMP(finalpan, -128, 127);
-
 	xc->info_finalpan = finalpan + 0x80;
 
 	if (xc->pan.surround) {
@@ -878,16 +914,16 @@ static void update_frequency(struct context_data *ctx, int chn)
 				  note_to_period(48, xc->finetune, 0, 0));
 	}
 
+	xc->arpeggio.count++;
+	xc->arpeggio.count %= xc->arpeggio.size;
+
 	/* Check for invalid periods (from Toru Egashira's NSPmod)
 	 * panic.s3m has negative periods
 	 * ambio.it uses low (~8) period values
 	 */
-	if (xc->period < 1) {
-		xc->volume = 0;
+	if (xc->period < 0.25) {
+		virt_setvol(ctx, chn, 0);
 	}
-
-	xc->arpeggio.count++;
-	xc->arpeggio.count %= xc->arpeggio.size;
 }
 
 static void update_pan(struct context_data *ctx, int chn)
@@ -1035,12 +1071,14 @@ static void next_order(struct context_data *ctx)
 	struct flow_control *f = &p->flow;
 	struct module_data *m = &ctx->m;
 	struct xmp_module *mod = &m->mod;
+	int mark;
 
 	do {
     		p->ord++;
 
 		/* Restart module */
-		if (p->ord >= mod->len || mod->xxo[p->ord] == 0xff) {
+		mark = HAS_QUIRK(QUIRK_MARKER) && mod->xxo[p->ord] == 0xff;
+		if (p->ord >= mod->len || mark) {
 			if (mod->rst > mod->len ||
 			    mod->xxo[mod->rst] >= mod->pat ||
 			    p->ord < m->seq_data[p->sequence].entry_point) {
@@ -1303,8 +1341,13 @@ int xmp_play_frame(xmp_context opaque)
 	if (ctx->state < XMP_STATE_PLAYING)
 		return -XMP_ERROR_STATE;
 
-	if (mod->len <= 0 || mod->xxo[p->ord] == 0xff)
+	if (mod->len <= 0) {
 		return -XMP_END;
+	}
+
+	if (HAS_QUIRK(QUIRK_MARKER) && mod->xxo[p->ord] == 0xff) {
+		return -XMP_END;
+	}
 
 	/* check reposition */
 	if (p->ord != p->pos) {
