@@ -34,19 +34,6 @@
 #endif
 
 
-static inline void copy_channel(struct player_data *p, int to, int from)
-{
-	if (to > 0 && to != from) {
-		memcpy(&p->xc_data[to], &p->xc_data[from],
-					sizeof (struct channel_data));
-	}
-}
-
-static inline int has_note_event(struct xmp_event *e)
-{
-	return (e->note && e->note <= XMP_MAX_KEYS);
-}
-
 static inline int is_valid_note(int note)
 {
 	return (note >= 0 && note < XMP_MAX_KEYS);
@@ -869,8 +856,7 @@ static int read_event_st3(struct context_data *ctx, struct xmp_event *e, int chn
 		xc->volume = sub->vol;
 	}
 
-	/* ST3: check QUIRK_ST3GVOL only in ST3 event reader */
-	if (HAS_QUIRK(QUIRK_ST3GVOL) && TEST(NEW_VOL)) {
+	if (HAS_QUIRK(QUIRK_ST3BUGS) && TEST(NEW_VOL)) {
 		xc->volume = xc->volume * p->gvol / m->volbase;
 	}
 
@@ -879,6 +865,19 @@ static int read_event_st3(struct context_data *ctx, struct xmp_event *e, int chn
 
 #ifndef LIBXMP_CORE_DISABLE_IT
 
+static inline void copy_channel(struct player_data *p, int to, int from)
+{
+	if (to > 0 && to != from) {
+		memcpy(&p->xc_data[to], &p->xc_data[from],
+					sizeof (struct channel_data));
+	}
+}
+
+static inline int has_note_event(struct xmp_event *e)
+{
+	return (e->note && e->note <= XMP_MAX_KEYS);
+}
+
 static int check_fadeout(struct context_data *ctx, struct channel_data *xc, int ins)
 {
 	struct xmp_instrument *xxi = get_instrument(ctx, ins);
@@ -1067,7 +1066,8 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 		if (IS_VALID_INSTRUMENT(ins)) {
 			/* valid ins */
 
-			if (!key) {
+			/* See OpenMPT StoppedInstrSwap.it for cut case */
+			if (!key && !TEST_NOTE(NOTE_KEY_CUT)) {
 				/* Retrig in new ins in sample mode */
 				if (sample_mode && TEST_NOTE(NOTE_END)) {
 					virt_voicepos(ctx, chn, 0);
@@ -1123,7 +1123,7 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 			reset_env = 0;
 			use_ins_vol = 0;
 		} else if (key == XMP_KEY_CUT) {
-			SET_NOTE(NOTE_END | NOTE_CUT);
+			SET_NOTE(NOTE_END | NOTE_CUT | NOTE_KEY_CUT);
 			xc->period = 0;
 			virt_resetchannel(ctx, chn);
 		} else if (key == XMP_KEY_OFF) {
@@ -1161,6 +1161,9 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 	}
 
 	if (is_valid_note(key - 1) && !new_invalid_ins) {
+		if (TEST_NOTE(NOTE_CUT)) {
+			use_ins_vol = 1;	/* See OpenMPT NoteOffInstr.it */
+		}
 		xc->key = --key;
 		RESET_NOTE(NOTE_END);
 
@@ -1169,6 +1172,7 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 		if (sub != NULL) {
 			int transp = mod->xxi[candidate_ins].map[key].xpo;
 			int smp, to;
+			int rvv;
 
 			note = key + sub->xpo + transp;
 			smp = sub->sid;
@@ -1183,6 +1187,24 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 			to = virt_setpatch(ctx, chn, candidate_ins, smp,
 				note, sub->nna, sub->dct, sub->dca);
 
+			/* Random value for volume swing */
+			rvv = sub->rvv & 0xff;
+			if (rvv) {
+				CLAMP(rvv, 0, 100);
+				xc->rvv = rand() % (rvv + 1);
+			} else {
+				xc->rvv = 0;
+			}
+
+			/* Random value for pan swing */
+			rvv = (sub->rvv & 0xff00) >> 8;
+			if (rvv) {
+				CLAMP(rvv, 0, 64);
+				xc->rpv = rand() % (rvv + 1) - 32;
+			} else {
+				xc->rpv = 0;
+			}
+
 			if (to < 0)
 				return -1;
 			if (to != chn) {
@@ -1242,9 +1264,10 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 	set_effect_defaults(ctx, note, sub, xc, is_toneporta);
 	if (sub != NULL) {
 		if (note >= 0) {
-			int pan_swing = (sub->rvv & 0xff00) >> 8;
-			CLAMP(pan_swing, 0, 64);
+			//int pan_swing = (sub->rvv & 0xff00) >> 8;
+			//CLAMP(pan_swing, 0, 64);
 
+			/* Reset pan, see OpenMPT PanReset.it */
 			if (sub->pan >= 0) {
 				xc->pan.val = sub->pan;
 				xc->pan.surround = 0;
@@ -1257,20 +1280,19 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 			}
 			RESET_NOTE(NOTE_CUT);
 
-			if (pan_swing) {
-				xc->pan.val = (xc->pan.val * (64 - pan_swing) +
-					(rand() % 256) * pan_swing) >> 6;
-			}
-		} else if (ev.ins) {
-			if (sub->pan >= 0)
-				xc->pan.val = sub->pan;
+			//if (pan_swing) {
+			//	xc->pan.val = (xc->pan.val * (64 - pan_swing) +
+			//		((rand() % 256) * pan_swing)) >> 6;
+			//}
 		}
 	}
 	
 	/* Process new volume */
 	if (ev.vol && (!TEST_NOTE(NOTE_CUT) || ev.ins != 0)) {
-		xc->volume = ev.vol - 1;
-		SET(NEW_VOL);
+		if (key != XMP_KEY_OFF) {    /* See OpenMPT NoteOffInstr.it */
+			xc->volume = ev.vol - 1;
+			SET(NEW_VOL);
+		}
 	}
 
 	/* IT: always reset sample offset */
@@ -1293,15 +1315,7 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 	}
 
 	if (use_ins_vol && !TEST(NEW_VOL)) {
-		int vol_swing = sub->rvv & 0xff;
-		CLAMP(vol_swing, 0, 100);
-
 		xc->volume = sub->vol;
-
-		if (vol_swing) {
-			xc->volume = (xc->volume * (100 - vol_swing) + 
-				(rand() % (xc->volume + 1)) * vol_swing) / 100;
-		}
 	}
 
 	return 0;
