@@ -64,6 +64,7 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
     int loop_row[XMP_MAX_CHANNELS];
     struct xmp_event* event;
     int i, pat;
+    int has_marker;
     struct ord_data *info;
 #ifndef LIBXMP_CORE_PLAYER
     int st26_speed;
@@ -94,6 +95,8 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
     st26_speed = 0;
 #endif
 
+    has_marker = HAS_QUIRK(QUIRK_MARKER);
+
     /* By erlk ozlr <erlk.ozlr@gmail.com>
      *
      * xmp doesn't handle really properly the "start" option (-s for the
@@ -130,7 +133,7 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 	    }
 	   
 	    pat = mod->xxo[ord];
-	    if (pat == S3M_END) {
+	    if (has_marker && pat == S3M_END) {
 		break;
 	    }
 	} 
@@ -146,7 +149,7 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 
 	/* All invalid patterns skipped, only S3M_END aborts replay */
 	if (pat >= mod->pat) {
-	    if (pat == S3M_END) {
+	    if (has_marker && pat == S3M_END) {
 		ord = mod->len;
 	        continue;
 	    }
@@ -370,6 +373,11 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 	    		m->scan_cnt[ord][row] += p1 & 0x0f;
 			frame_count += (p1 & 0x0f) * speed;
 		}
+
+		if (f1 == FX_IT_BREAK) {
+		    break_row = p1;
+		    last_row = 0;
+		}
 #endif
 
 		if (f1 == FX_JUMP || f2 == FX_JUMP) {
